//**********************************************************************
// 	Project: 
//	File: RS_exp.v
// 	Description: test RS Encoder / Decoder （GW_2A_18）
//	Author: Wx
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	|           	| xx/06/2023	    | Initial version
//**********************************************************************


module RS_exp(
// --- sym ---
	input    SYM_CLK,
// --- KEY ---
	input    KEY1,
// --- SW ----
	input    SW1,

    output [7:0] Decoder_data_out,
    output sop_out,
    output eop_out,
    output Decoder_outvalid,
    output  [7:0] error_number
);

//**********************************************************************
// --- Parameter
//**********************************************************************
    localparam NUM_CYCLE = 1;
	localparam MEM_DEPTH = 8*8;

//**********************************************************************
// --- Inter Signal Declaration
//**********************************************************************
    wire clk;
    // wire clk_100M;
	wire rstn;
	wire ce;
    assign clk = SYM_CLK;
    assign rstn = KEY1;
	assign ce=SW1;

    reg Encoder_inpvalid;
wire Decoder_inpvalid;
    reg [6:0] cycle_cnt,cycle_cnt2;
    reg [7:0] din;
    wire [7:0] Encoder_data_out;
    wire Encoder_outvalid;
    reg [7:0]  noise_code;
    wire [7:0]  Decoder_data_in;
    wire sop_in,eop_in;
    reg [7:0] noise;
    wire Enc_sop_in,Enc_eop_in;
    wire Enc_sop_out,Enc_eop_out;
//**********************************************************************
// --- Initial
//**********************************************************************
//******Encoder***************//

always@(posedge clk or negedge rstn) begin
    if(!rstn)
        cycle_cnt<=0;
    else 
        cycle_cnt<=cycle_cnt+1;
end

always@(posedge clk or negedge rstn) begin
    if(!rstn)
        din<=0;
    else if((cycle_cnt>=1)&&(cycle_cnt<=28))
        din<=din+1;
    else
       din<=0;

end

always@(posedge clk or negedge rstn) begin
    if(!rstn)
        Encoder_inpvalid<=0;
    else if(cycle_cnt==1)
        Encoder_inpvalid<=1;
    else if(cycle_cnt==8'd29)
        Encoder_inpvalid<=0;
end
assign Enc_sop_in=(cycle_cnt==2) ? 1 : 0;
assign Enc_eop_in=(cycle_cnt==29) ? 1 : 0;

//*****Decoder*****************
always@(posedge clk or negedge rstn) begin  //change any number
    if(!rstn)
        noise<=0;
    else if(cycle_cnt==9)
        noise<=noise+6;
    else if   (cycle_cnt==14)
        noise<=noise+5;
    else 
        noise<=0;
end

//*****Decoder*****************
assign Decoder_data_in=noise^Encoder_data_out;
assign sop_in=Enc_sop_out;
assign eop_in=Enc_eop_out;
assign Decoder_inpvalid=Encoder_outvalid;
//**********************************************************************
// --- Module: top 
// --- Description:	top module of project
//**********************************************************************
RS_Encoder_Top RS_Encoder_Top(
    .clk        (clk), 
    .rstn       (rstn),
    .sop_in     (Enc_sop_in),
    .eop_in     (Enc_eop_in),
    .inpvalid   (Encoder_inpvalid),
    .data_in    (din),
    .sop_out    (Enc_sop_out),
    .eop_out    (Enc_eop_out),    
    .outvalid   (Encoder_outvalid),
    .data_out   (Encoder_data_out),
    .ready_in   (ready_in)
    );


RS_Decoder_Top RS_Decoder_Top(
    .clk        (clk),
    .rstn       (rstn),
    .inpvalid   (Decoder_inpvalid),
    .sop_in     (sop_in),
    .eop_in     (eop_in),
    .data_in    (Decoder_data_in),
    .sop_out    (sop_out),
    .eop_out    (eop_out),
    .outvalid   (Decoder_outvalid),
    .data_out   (Decoder_data_out),
    .error_number(error_number)
);


	
 
endmodule
